==> fred_verifier-0.1.0-py3-none-any.whl: verifier/client.py <==
"""Verification service client."""
from typing import Iterable, List, cast

import fred_api.verification.service_verification_grpc_pb2_grpc
from fred_api.verification.common_types_pb2 import (
    ContactDoesNotExist as ContactDoesNotExistMessage,
    Event as EventMessage,
    MethodNotAvailable as MethodNotAvailableMessage,
    PlaceAddress,
    VerificationDoesNotExist as VerificationDoesNotExistMessage,
    VerificationInfo,
    VerificationStatus as VerificationEnum,
)
from fred_api.verification.pagination_pb2 import PaginationReply
from fred_api.verification.service_verification_grpc_pb2 import (
    CancelRequest,
    CreateRequest,
    GetContactRepresentativeRequest,
    GetMethodsRequest,
    GetRequest,
    ListRequest,
    MarkVerifiedRequest,
    SetContactRepresentativeRequest,
    SetMethodRequest,
    VerifyTokenRequest,
)
from frgal import GrpcDecoder
from frgal.aio import AsyncGrpcClient
from regal import Address, ContactId

from .common import (
    ContactData,
    ContactRepresentative,
    Event,
    PaginatedList,
    Pagination,
    Verification,
    VerificationId,
)
from .exceptions import (
    ContactDoesNotExist,
    MethodNotAvailable,
    VerificationDoesNotExist,
)


class VerificationDecoder(GrpcDecoder):
    """Registrar service decoder."""

    decode_unset_messages = False
    decode_empty_string_none = False

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(EventMessage, self._decode_event)
        self.set_decoder(VerificationInfo, self._decode_verification)
        self.set_decoder(PaginationReply, self._decode_pagination)
        self.set_decoder(PlaceAddress, self._decode_address)

        self.set_exception_decoder(ContactDoesNotExistMessage, ContactDoesNotExist)
        self.set_exception_decoder(
            VerificationDoesNotExistMessage, VerificationDoesNotExist
        )
        self.set_exception_decoder(MethodNotAvailableMessage, MethodNotAvailable)

    def _decode_event(self, value: EventMessage) -> Event:
        dict_value = self._decode_message(value)
        if dict_value["details"] is None:
            dict_value.pop("details")
        return Event(**dict_value)

    def _decode_verification(self, value: VerificationInfo) -> Verification:
        dict_value = self._decode_message(value)
        dict_value.update({"status": VerificationEnum.Name(value.status)})
        return Verification(**dict_value)

    def _decode_pagination(self, value: PaginationReply) -> Pagination:
        return Pagination(**self._decode_message(value))

    def _decode_address(self, value: PlaceAddress) -> Address:
        return Address(**self._decode_message(value))


class VerificationClient(AsyncGrpcClient):
    """Verification service gRPC client."""

    service = "Verification"
    decoder_cls = VerificationDecoder
    grpc_modules = (fred_api.verification.service_verification_grpc_pb2_grpc,)

    async def create(self, contact_id: ContactId) -> Verification:
        """Create verification for contact.

        Args:
            contact_id: Contact id.

        Returns:
            Verification info.

        Raises:
            ContactDoesNotExist: If contact does not exist.
        """
        request = CreateRequest()
        request.contact_id.value = contact_id
        return cast(Verification, await self.call(self.service, "create", request))

    async def cancel(self, verification_id: VerificationId) -> None:
        """Cancel verification.

        Args:
            verification_id: Verification id.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = CancelRequest()
        request.verification_id.value = verification_id
        await self.call(self.service, "cancel", request)

    async def get(self, verification_id: VerificationId) -> Verification:
        """Get verification info.

        Args:
            verification_id: Verification id.

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetRequest()
        request.verification_id.value = verification_id
        return cast(Verification, await self.call(self.service, "get", request))

    async def list(
        self,
        contact_id: ContactId = None,
        include_closed: bool = False,
        *,
        page_token: str = None,
        page_size: int = None,
    ) -> PaginatedList[Verification]:
        """List verifications.

        Args:
            contact_id: When set, filter verifications by contact_id.
            include_closed: If True, include closed verifications in the result.
            page_token: Page token for pagination.
            page_size: Page size for pagination.

        Returns:
            Paginated list of verifications.
        """
        request = ListRequest()
        if contact_id is not None:
            request.contact_id.value = contact_id
        request.include_closed = include_closed
        if page_token is not None:
            request.pagination.page_token = page_token
        if page_size is not None:
            request.pagination.page_size = page_size

        result: PaginatedList[Verification] = PaginatedList(results=[])
        async for message in self.call_stream(
            self.service, "list", request
        ):  # pragma: no cover
            if message:
                if result.pagination is None:
                    result.pagination = message.get("pagination")
                result.results.extend(message.get("verifications", []))

        return result

    async def get_methods(self, verification_id: VerificationId) -> List[str]:
        """Get available methods for verification.

        Args:
            verification_id: Verification id.

        Returns:
            List of available methods.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetMethodsRequest()
        request.verification_id.value = verification_id
        result = await self.call(self.service, "get_methods", request)
        return cast(List[str], result)

    async def set_method(
        self, verification_id: VerificationId, method: str
    ) -> Verification:
        """Set chosen method for verification.

        Args:
            verification_id: Verification id.
            method: Chosen method name.

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            MethodNotAvailable: If method is not available for verification.
        """
        request = SetMethodRequest()
        request.verification_id.value = verification_id
        request.method = method
        result = await self.call(self.service, "set_method", request)
        return cast(Verification, result)

    async def mark_verified(
        self,
        verification_id: VerificationId,
        *,
        method: str = None,
        verified_data: Iterable[ContactData] = None,
    ) -> Verification:
        """Mark selected contact data as verified.

        Args:
            verification_id: Verification id.
            method: Verification method used to verify the data.
            verified_data: Iterable of verified data (contact fields).

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verificaiton does not exist.
            MethodNotAvailable: If method is not available for verification.
        """
        request = MarkVerifiedRequest()
        request.verification_id.value = verification_id
        if method is not None:
            request.method = method
        for field in verified_data or ():
            setattr(request, field, True)

        return cast(
            Verification, await self.call(self.service, "mark_verified", request)
        )

    async def verify_token(self, verification_id: VerificationId, token: str) -> bool:
        """Verify token for a verification.

        Args:
            verification_id: Verification id.
            token: Token.

        Returns:
            Boolean indicating whether token is valid for the verification.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = VerifyTokenRequest()
        request.verification_id.value = verification_id
        request.token = token
        return cast(bool, await self.call(self.service, "verify_token", request))

    async def get_contact_representative(
        self, verification_id: VerificationId
    ) -> ContactRepresentative:
        """Get contact representative.

        Args:
            verification_id: Verification id.

        Returns:
            Contact representative.

        Raises:
            VerificationDoesNotExists: If verification does not exist.
        """
        request = GetContactRepresentativeRequest()
        request.verification_id.value = verification_id
        return ContactRepresentative(
            **await self.call(self.service, "get_contact_representative", request)
        )

    async def set_contact_representative(
        self, verification_id: VerificationId, representative: ContactRepresentative
    ) -> None:
        """Set contact representative.

        Args:
            verification_id: Verification id.
            representative: Contact representative.

        Raises:
            VerificationDoesNotExists: If verification does not exist.
        """
        request = SetContactRepresentativeRequest()
        request.verification_id.value = verification_id
        request.organization = representative.organization
        request.address.street[:] = representative.address.street
        request.address.city = representative.address.city
        request.address.state_or_province = representative.address.state_or_province
        request.address.postal_code = representative.address.postal_code
        request.address.country_code = representative.address.country_code
        await self.call(self.service, "set_contact_representative", request)

==> fred_verifier-0.2.0-py3-none-any.whl: verifier/client.py <==
"""Verification service client."""
from typing import List, Tuple, cast

import fred_api.verification.service_verification_grpc_pb2_grpc
from fred_api.verification.common_types_pb2 import (
    ContactDoesNotExist as ContactDoesNotExistMessage,
    Event as EventMessage,
    MethodNotAvailable as MethodNotAvailableMessage,
    PlaceAddress,
    VerificationDoesNotExist as VerificationDoesNotExistMessage,
    VerificationInfo,
    VerificationStatus as VerificationEnum,
)
from fred_api.verification.pagination_pb2 import PaginationReply
from fred_api.verification.service_verification_grpc_pb2 import (
    CancelRequest,
    CreateRequest,
    GetContactRepresentativeRequest,
    GetMethodsRequest,
    GetRequest,
    ListContactDataVerificationsRequest,
    ListRequest,
    SetContactRepresentativeRequest,
    UseMethodRequest,
    VerifyContactDataRequest,
    VerifyTokenRequest,
)
from frgal import GrpcDecoder
from frgal.aio import AsyncGrpcClient
from regal import Address, ContactId

from .common import (
    ContactData,
    ContactRepresentative,
    Event,
    PaginatedList,
    Pagination,
    Verification,
    VerificationId,
    VerificationResult,
)
from .exceptions import (
    ContactDoesNotExist,
    MethodNotAvailable,
    VerificationDoesNotExist,
)


class VerificationDecoder(GrpcDecoder):
    """Registrar service decoder."""

    decode_unset_messages = False
    decode_empty_string_none = False

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(EventMessage, self._decode_event)
        self.set_decoder(VerificationInfo, self._decode_verification)
        self.set_decoder(PaginationReply, self._decode_pagination)
        self.set_decoder(PlaceAddress, self._decode_address)

        self.set_exception_decoder(ContactDoesNotExistMessage, ContactDoesNotExist)
        self.set_exception_decoder(
            VerificationDoesNotExistMessage, VerificationDoesNotExist
        )
        self.set_exception_decoder(MethodNotAvailableMessage, MethodNotAvailable)

    def _decode_event(self, value: EventMessage) -> Event:
        dict_value = self._decode_message(value)
        if dict_value["details"] is None:
            dict_value.pop("details")
        return Event(**dict_value)

    def _decode_verification(self, value: VerificationInfo) -> Verification:
        dict_value = self._decode_message(value)
        dict_value.update({"status": VerificationEnum.Name(value.status)})
        return Verification(**dict_value)

    def _decode_pagination(self, value: PaginationReply) -> Pagination:
        return Pagination(**self._decode_message(value))

    def _decode_address(self, value: PlaceAddress) -> Address:
        return Address(**self._decode_message(value))


class VerificationClient(AsyncGrpcClient):
    """Verification service gRPC client."""

    service = "Verification"
    decoder_cls = VerificationDecoder
    grpc_modules = (fred_api.verification.service_verification_grpc_pb2_grpc,)

    async def create(self, contact_id: ContactId) -> Verification:
        """Create verification for contact.

        Args:
            contact_id: Contact id.

        Returns:
            Verification info.

        Raises:
            ContactDoesNotExist: If contact does not exist.
        """
        request = CreateRequest()
        request.contact_id.value = contact_id
        return cast(Verification, await self.call(self.service, "create", request))

    async def cancel(self, verification_id: VerificationId) -> None:
        """Cancel verification.

        Args:
            verification_id: Verification id.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = CancelRequest()
        request.verification_id.value = verification_id
        await self.call(self.service, "cancel", request)

    async def get(self, verification_id: VerificationId) -> Verification:
        """Get verification info.

        Args:
            verification_id: Verification id.

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetRequest()
        request.verification_id.value = verification_id
        return cast(Verification, await self.call(self.service, "get", request))

    async def list(
        self,
        contact_id: ContactId = None,
        include_closed: bool = False,
        *,
        page_token: str = None,
        page_size: int = None,
    ) -> PaginatedList[Verification]:
        """List verifications.

        Args:
            contact_id: When set, filter verifications by contact_id.
            include_closed: If True, include closed verifications in the result.
            page_token: Page token for pagination.
            page_size: Page size for pagination.

        Returns:
            Paginated list of verifications.
        """
        request = ListRequest()
        if contact_id is not None:
            request.contact_id.value = contact_id
        request.include_closed = include_closed
        if page_token is not None:
            request.pagination.page_token = page_token
        if page_size is not None:
            request.pagination.page_size = page_size

        result: PaginatedList[Verification] = PaginatedList(results=[])
        async for message in self.call_stream(
            self.service, "list", request
        ):  # pragma: no cover
            if message:
                if result.pagination is None:
                    result.pagination = message.get("pagination")
                result.results.extend(message.get("verifications", []))

        return result

    async def get_methods(self, verification_id: VerificationId) -> List[str]:
        """Get available methods for verification.

        Args:
            verification_id: Verification id.

        Returns:
            List of available methods.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetMethodsRequest()
        request.verification_id.value = verification_id
        result = await self.call(self.service, "get_methods", request)
        return cast(List[str], result)

    async def use_method(
        self, verification_id: VerificationId, method: str
    ) -> Verification:
        """Use chosen method for verification.

        Args:
            verification_id: Verification id.
            method: Chosen method name.

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            MethodNotAvailable: If method is not available for verification.
        """
        request = UseMethodRequest()
        request.verification_id.value = verification_id
        request.method = method
        result = await self.call(self.service, "use_method", request)
        return cast(Verification, result)

    async def verify_token(self, verification_id: VerificationId, token: str) -> bool:
        """Verify token for a verification.

        Args:
            verification_id: Verification id.
            token: Token.

        Returns:
            Boolean indicating whether token is valid for the verification.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = VerifyTokenRequest()
        request.verification_id.value = verification_id
        request.token = token
        return cast(bool, await self.call(self.service, "verify_token", request))

    async def verify_contact_data(
        self, verification_id: VerificationId, method: str, data: ContactData
    ) -> Tuple[Verification, VerificationResult]:
        """Verify contact data.

        Args:
            verification_id: Verification id.
            method: Verification method.
            data: Contact data to verify.

        Returns:
            Updated verification and verification result.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            MethodNotAvailable: If method is not available for verification.
        """
        request = VerifyContactDataRequest()
        request.verification_id.value = verification_id
        request.method = method
        if data.name is not None:
            request.data.name = data.name
        if data.organization is not None:
            request.data.organization = data.organization
        if data.birthdate is not None:
            request.data.birthdate = data.birthdate.isoformat()
        if data.company_registration_number is not None:
            request.data.company_registration_number = data.company_registration_number
        if data.address is not None:
            request.data.address.street[:] = data.address.street
            request.data.address.city = data.address.city
            request.data.address.state_or_province = data.address.state_or_province
            request.data.address.postal_code = data.address.postal_code
            request.data.address.country_code = data.address.country_code

        result = await self.call(self.service, "verify_contact_data", request)
        return (
            result.get("verification"),
            VerificationResult(**result.get("result")),
        )

    async def list_contact_data_verifications(
        self, verification_id: VerificationId
    ) -> List[VerificationResult]:
        """Get list of contact data verifications.

        Args:
            verification_id: Verification id.

        Returns:
            List of contact data verifications.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = ListContactDataVerificationsRequest()
        request.verification_id.value = verification_id

        verifications = await self.call(
            self.service, "list_contact_data_verifications", request
        )
        return [VerificationResult(**result) for result in verifications]

    async def get_contact_representative(
        self, verification_id: VerificationId
    ) -> ContactRepresentative:
        """Get contact representative.

        Args:
            verification_id: Verification id.

        Returns:
            Contact representative.

        Raises:
            VerificationDoesNotExists: If verification does not exist.
        """
        request = GetContactRepresentativeRequest()
        request.verification_id.value = verification_id
        return ContactRepresentative(
            **await self.call(self.service, "get_contact_representative", request)
        )

    async def set_contact_representative(
        self, verification_id: VerificationId, representative: ContactRepresentative
    ) -> None:
        """Set contact representative.

        Args:
            verification_id: Verification id.
            representative: Contact representative.

        Raises:
            VerificationDoesNotExists: If verification does not exist.
        """
        request = SetContactRepresentativeRequest()
        request.verification_id.value = verification_id
        request.organization = representative.organization
        request.address.street[:] = representative.address.street
        request.address.city = representative.address.city
        request.address.state_or_province = representative.address.state_or_province
        request.address.postal_code = representative.address.postal_code
        request.address.country_code = representative.address.country_code
        await self.call(self.service, "set_contact_representative", request)
